﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Collections.ObjectModel;
using Infragistics.Windows.DataPresenter;
using Infragistics.DragDrop;
using Infragistics.Windows;
using System.Diagnostics;

namespace DragDropRecordsInHierarchy
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();

            Data = new ObservableCollection<Parent>();
            for (int i = 0; i < 100; i++)
            {
                Data.Add(new Parent
                {
                    ID = i,
                    Name = "Parent " + i,
                });
            }

            DataContext = Data;
        }

        public ObservableCollection<Parent> Data { get; set; }

     
        private void DragSource_Drop(object sender, DropEventArgs e)
        {            
            if (e.Data is ObservableCollection<Parent>)
            {
                Parent drag = ((e.DragSource as ContentPresenter).DataContext as DataRecord).DataItem as Parent;
                Parent drop = ((e.DropTarget as ContentPresenter).DataContext as DataRecord).DataItem as Parent;

                ObservableCollection<Parent> parentCollection = e.Data as ObservableCollection<Parent>;

                int dragIndex = parentCollection.IndexOf(drag);
                int dropIndex = parentCollection.IndexOf(drop);

                parentCollection.Remove(drag);
                parentCollection.Insert(dropIndex, drag);
            } 
        }

        private void DragSource_DragStart(object sender, DragDropStartEventArgs e)
        {
            e.Data = (Utilities.GetAncestorFromType(e.DragSource, typeof(DataRecordPresenter), false) as DataRecordPresenter).Record.DataPresenter.DataSource;
        }

        private void DragSource_DragOver(object sender, DragDropMoveEventArgs e)
        {
            DataRecordPresenter presenter = Utilities.GetAncestorFromType(e.DragSource, typeof(DataRecordPresenter), false) as DataRecordPresenter;
            if (presenter != null)
            {
                XamDataGrid grid = presenter.Record.DataPresenter as XamDataGrid;
                ScrollViewer scrl = Utilities.GetDescendantFromType(grid, typeof(ScrollViewer), false) as ScrollViewer;

                Point p = e.GetPosition(grid);
                bool shouldScroll = p.Y > grid.ActualHeight - 10;

                if (scrl != null)
                { 
                    if (p.Y < 50)
                    {
                        scrl.ScrollToVerticalOffset(scrl.VerticalOffset - 1);
                    }
                    if (p.Y > grid.ActualHeight - 10)
                    {
                        scrl.ScrollToVerticalOffset(scrl.VerticalOffset + 1);                        
                    }
                }
            }
        }
    }


    public class Parent
    {
        public int ID { get; set; }
        public string Name { get; set; } 
    }
     

    public class DepthToDragDropChannels : IValueConverter
    {
        public object Convert(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            return new ObservableCollection<string> { value.ToString() };
        }

        public object ConvertBack(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            throw new NotImplementedException();
        }
    }

}
